@@ -66,7 +66,6 @@ namespace Soprano {
 
         int usedPort() const;
 
-        bool isRunning() const;
         Status status() const { return m_status; }
         ExitStatus lastExitStatus() const { return m_lastExitStatus; }
 
@@ -75,7 +74,7 @@ namespace Soprano {
 
     Q_SIGNALS:
         void started();
-        void stopped( ExitStatus status );
+        void stopped( Soprano::VirtuosoController::ExitStatus status );
 
     private Q_SLOTS:
         void slotProcessFinished( int exitCode, QProcess::ExitStatus exitStatus );
